//Microchip Solutions v2012-04-03
/******************************************************************************
            USB Custom Demo, Host

This file provides the main entry point to the Microchip USB Custom
Host demo.  This demo shows how a PIC24F system could be used to
act as the host, controlling a USB device running the Microchip Custom
Device demo.

******************************************************************************/

/******************************************************************************
* Filename:        main.c
* Dependancies:    USB Host Driver with Generic Client Driver
* Processor:       PIC24F256GB1xx
* Hardware:        Explorer 16 with USB PICtail Plus
* Compiler:        C30 v2.01/C32 v0.00.18
* Company:         Microchip Technology, Inc.

Software License Agreement

The software supplied herewith by Microchip Technology Incorporated
(the CompanyE for its PICmicro Microcontroller is intended and
supplied to you, the Companys customer, for use solely and
exclusively on Microchip PICmicro Microcontroller products. The
software is owned by the Company and/or its supplier, and is
protected under applicable copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to
civil liability for the breach of the terms and conditions of this
license.

THIS SOFTWARE IS PROVIDED IN AN AS ISECONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.


*******************************************************************************/

#include <stdlib.h>
#include "GenericTypeDefs.h"
#include "HardwareProfile.h"
#include "usb_config.h"
#include "USB/usb.h"
#include "USB/usb_host_generic.h"
#include "user.h"
#include "timer.h"

void Write_ACL_Command(char *, int);
void Write_HCI_Command(char *, int);

void hci_acl_init(void);
void hid(void);

//#define DEBUG_MODE
// *****************************************************************************
// *****************************************************************************
// Configuration Bits
// *****************************************************************************
// *****************************************************************************

        _CONFIG1(WDTPS_PS1 & FWPSA_PR32 & WINDIS_OFF & FWDTEN_OFF & ICS_PGx1 & GWRP_OFF & GCP_OFF & JTAGEN_OFF)
        _CONFIG2(POSCMOD_HS & I2C1SEL_PRI & IOL1WAY_OFF & OSCIOFNC_ON & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_NODIV & IESO_ON)
        _CONFIG3(WPFP_WPFP0 & SOSCSEL_IO & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM)
        _CONFIG4(DSWDTPS_DSWDTPS3 & DSWDTOSC_LPRC & RTCOSC_SOSC & DSBOREN_OFF & DSWDTEN_OFF)

// *****************************************************************************

// Application States
typedef enum
{
    BT_INITIALIZE = 0,	// Initialize the app when a device is attached
    BT_STATE_IDLE,
    BT_STATE_PROCESS,
    BT_STATE_WAIT_DATA
} BT_STATE;

// Hci States
typedef enum
{
	HCI_CMD_RESET = 0,	// Initialize the hci when a device is attached
	HCI_CMD_READ_BD_ADDR,
	HCI_CMD_LOCAL_NAME,
	HCI_CMD_CLASS_DEVICE,
	HCI_CMD_SCAN_ENABLE,
	HCI_CMD_WAIT_CONNECTION,
	HCI_CMD_CONNECTION_ACCEPT,
	HCI_CMD_DISCONNECT_REQ,

	HCI_END,
} HCI_STATE;


// *****************************************************************************
// *****************************************************************************
// Global Variables
// *****************************************************************************
// *****************************************************************************

BYTE	deviceAddress;	// Address of the device on the USB
BT_STATE  DemoState;	// Current state of the demo application
HCI_STATE  HciState;	// Current state of the demo application

WORD data_size;
int data_num;

unsigned char buf[64];
unsigned char buf1[64];
unsigned char handle[2];

unsigned char ConnectSwitch;
unsigned char hci_busy_flag=0;
unsigned char acl_busy_flag=0;

//******************************************************************************
//******************************************************************************
// Local Routines
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        InitializeSystem
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         TRUE if successful, FALSE if not.
 *
 * Side Effects:    See below
 *
 * Overview:        This routine initializes the processor and peripheral,
 *                  setting clock speeds and enabling any required
 *                  features.
 *************************************************************************/

BOOL InitializeSystem ( void )
{

	unsigned int pll_startup_counter = 600;
	CLKDIVbits.PLLEN = 1;
	while(pll_startup_counter--);


	// Configure U2RX - put on pin 17 (RP8)
	RPINR19bits.U2RXR = 8;
	// Configure U2TX - put on pin 16 (RP7)
	RPOR3bits.RP7R = 5;
    // Init UART
    UART2Init();

	AD1PCFG = 0xFFFF; /*set AN pins to digital pin */
	TRISA=0x0003;	/* set PORTB(RA0 and RA1 pins) to input*/
					/* "0" corresponds to "o"utput and "1" to "i"nput */
	TRISB=0x001C;	/* set PORTB(RB2 and RB4 pins) to input*/
					/* "0" corresponds to "o"utput and "1" to "i"nput */
					/* SOSCSEL_IO setting in CONFIG3 to use RA4 and RB4 pins as digital ones */ 
	CNPU1=0x00CE;	/* Pull up CN1, 2, 3, 6, 7, pins  */

    // Set Default demo state
    DemoState = BT_INITIALIZE;
	HciState=HCI_CMD_RESET;

	ConnectSwitch=PORTBbits.RB4;
ConnectSwitch=0;
    return TRUE;
} // InitializeSystem


/*************************************************************************
 * Function:        CheckForNewAttach
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          deviceAddress (global)
 *                  Updates the device address when an attach is found.
 *
 * Returns:         TRUE if a new device has been attached.  FALSE,
 *                  otherwise.
 *
 * Side Effects:    Prints attach message
 *
 * Overview:        This routine checks to see if a new device has been
 *                  attached.  If it has, it records the address.
 *************************************************************************/

BOOL CheckForNewAttach ( void )
{
    // Try to get the device address, if we don't have one.
    if (deviceAddress == 0)
    {
        GENERIC_DEVICE_ID DevID;

        DevID.vid   = 0x04D8;
        DevID.pid   = 0x000C;
        #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
            DevID.serialNumberLength = 0;
            DevID.serialNumber = NULL;
        #endif

        if (USBHostGenericGetDeviceAddress(&DevID))
        {
            deviceAddress = DevID.deviceAddress;
            UART2PrintString( "Generic demo device attached - polled, deviceAddress=" );
            UART2PutDec( deviceAddress );
            UART2PrintString( "\r\n" );
            return TRUE;
        }
    }

    return FALSE;

} // CheckForNewAttach


//********************************************************************************
//********************************************************************************
// Initialization for HCI and ACL
//********************************************************************************
//********************************************************************************
void hci_acl_init(void){
	switch (HciState)
	{
		case HCI_CMD_RESET:
		case HCI_CMD_READ_BD_ADDR:

			buf1[0]=0x09;
			buf1[1]=0x10;
			buf1[2]=0;

			Write_HCI_Command( "HCI_CMD_READ_BD_ADDR: " ,3);
			HciState = HCI_CMD_LOCAL_NAME;
			break;

		case HCI_CMD_LOCAL_NAME:

			buf1[0]=0x13;
			buf1[1]=0x0c;
			buf1[2]=0x04;
			buf1[3]='y';
			buf1[4]='t';
			buf1[5]='2';
			buf1[6]=0x00;

			Write_HCI_Command( "HCI_CMD_LOCAL_NAME: " ,7);
			HciState = HCI_CMD_CLASS_DEVICE;
			break;

		case HCI_CMD_CLASS_DEVICE:

			buf1[0]=0x24;
			buf1[1]=0x0c;
			buf1[2]=0x03;
			buf1[3]=0x04;//joystick
			buf1[4]=0x05;//joystick
			buf1[5]=0x00;

			Write_HCI_Command( "HCI_CMD_CLASS_DEVICE: " ,6);
			HciState =HCI_CMD_SCAN_ENABLE;
			break;

		case HCI_CMD_SCAN_ENABLE:

			buf1[0]=0x1a;
			buf1[1]=0x0c;
			buf1[2]=0x01;
			buf1[3]=0x03;

			Write_HCI_Command( "HCI_CMD_SCAN_ENABLE: " ,4);

			HciState = HCI_CMD_WAIT_CONNECTION;

			break;

//********************************************************************************
//********************************************************************************
// FIRST CONNECTION STAGE
//********************************************************************************
//********************************************************************************
		case HCI_CMD_WAIT_CONNECTION:

			HciState = HCI_CMD_CONNECTION_ACCEPT;
			break;

		case HCI_CMD_CONNECTION_ACCEPT:

			buf1[8]=buf1[7];//copy address
			buf1[7]=buf1[6];
			buf1[6]=buf1[5];
			buf1[5]=buf1[4];
			buf1[4]=buf1[3];
			buf1[3]=buf1[2];
			buf1[0]=0x09;
			buf1[1]=0x04;
			buf1[2]=0x07;
			buf1[9]=0x01;//role

			Write_HCI_Command( "HCI_CMD_CONNECTION_ACCEPT: " ,10);
			HciState =HCI_END;
			break;

//********************************************************************************
//********************************************************************************
		case HCI_CMD_DISCONNECT_REQ:
			buf1[0]=0x06;
			buf1[1]=0x04;
			buf1[2]=0x03;
			buf1[3]=handle[0];
			buf1[4]=handle[1]-0x20;
			buf1[5]=0x13;

			DemoState = BT_STATE_WAIT_DATA;
			Write_HCI_Command( "HCI_CMD_DISCONNECT_REQ: " ,6);
			HciState = HCI_CMD_CONNECTION_ACCEPT;
			break;

//********************************************************************************

		case  HCI_END:
			break;
	}
}


//********************************************************************************
//********************************************************************************
//HID layer
//********************************************************************************
//********************************************************************************
void hid(void){

			buf[2]=0x36;
			buf[3]=0x00;
			buf[4]=0x32;
			buf[5]=0x00;
			buf[6]=0x40;
			buf[7]=0x00;
			buf[8]+=1;

			acl_busy_flag=1;// Wait for ACL DATA
			Write_ACL_Command("  HID_WRITE1:\r\n      ", 58);
}

//********************************************************************************
//********************************************************************************
// WRITE functions
//********************************************************************************
//********************************************************************************

void Write_ACL_Command(char *message, int data_size2)
{

	hci_busy_flag=1; buf1[0]=0xff;

	#ifdef DEBUG_MODE
		UART2PrintString( message);
		for(data_num=0;data_num<data_size2;data_num++)
	      	 {UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
	#endif	

    if (!USBHostGenericTxIsBusy(deviceAddress))
	{
		USBHostGenericAclWrite( deviceAddress, buf, data_size2 );
	}
	else{
		UART2PrintString( "ACL cannot be written *********************** \r\n");
	}
}

void Write_HCI_Command(char *message, int data_size1)
{
	#ifdef DEBUG_MODE
		UART2PrintString( message);
		for(data_num=0;data_num<data_size1;data_num++)
	      	 {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
	#endif	

	if ( USBHostGenericClassRequest( deviceAddress, buf1, data_size1 ) == USB_SUCCESS )
	{
//	UART2PrintString( "HCI COMMAND SENT\r\n" );	
	}
	else
	{
	UART2PrintString( "Write Class Error !\r\n" );	
	}
}

/*************************************************************************
 * Function:        ManageDemoState
 *
 * Preconditions:   The DemoState global variable must be initialized to
 *                  DEMO_STATE_IDLE (0).  (This occurs on reset.)
 *
 * Input:           DemoState (global)
 *                  Actions selected based value of DemoState on function
 *                  entry.
 *
 *                  deviceAddress (global)
 *                  May use device address to access device, depending on
 *                  state.
 *
 *                  DataPacket (global)
 *                  May read data from packet buffer, depending on state.
 *
 * Output:          DemoState (global)
 *                  Updates demo state as appropriate.
 *
 *                  DataPacket (global)
 *                  May cause data in the packet buffer to be updated,
 *                  depending on state.
 *
 * Returns:         None
 *
 * Side Effects:    Depend on state transition
 *
 * Overview:        This routine maintains the state of the application,
 *                  updateing global data and taking actions as necessary
 *                  to maintain the custom demo operations.
 *************************************************************************/
void ManageDemoState ( void )
{

    // Watch for device detaching
    if (USBHostGenericDeviceDetached(deviceAddress) && deviceAddress != 0)
    {
        UART2PrintString( "Generic demo device detached - polled\r\n" );
        DemoState = BT_INITIALIZE;
        deviceAddress   = 0;
    }

    switch (DemoState)
    {
		case BT_INITIALIZE:
			DemoState = BT_STATE_IDLE;
        	break;

   	 /** Idle State:  Loops here until attach **/
	    case BT_STATE_IDLE:
      	  if (CheckForNewAttach())
      	  {
				DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET; 
      	  }
      	  break;

  	  case BT_STATE_PROCESS:

			buf1[0]=0x03;
			buf1[1]=0x0c;
			buf1[2]=0;

			Write_HCI_Command( "HCI_CMD_RESET: " ,3);

			DemoState = BT_STATE_WAIT_DATA;
			break;

    	case BT_STATE_WAIT_DATA:
			if (!USBHostGenericRx1IsBusy(deviceAddress) ) {
				USBHostGenericRead(deviceAddress,buf1,DATA_PACKET_LENGTH);
			}
			else{
				UART2PrintString( "HCI cannot be armed \r\n" );
			}
			if (!USBHostGenericRx2IsBusy(deviceAddress)) {
				USBHostGenericAclRead(deviceAddress, buf, DATA_PACKET_LENGTH);
			}
			break;
	}
    DelayMs(1); // 1ms delay

} // ManageDemoState


//******************************************************************************
//******************************************************************************
// USB Support Functions
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        USB_ApplicationEventHandler
 *
 * Preconditions:   The USB must be initialized.
 *
 * Input:           event       Identifies the bus event that occured
 *
 *                  data        Pointer to event-specific data
 *
 *                  size        Size of the event-specific data
 *
 * Output:          deviceAddress (global)
 *                  Updates device address when an attach or detach occurs.
 *
 *                  DemoState (global)
 *                  Updates the demo state as appropriate when events occur.
 *
 * Returns:         TRUE if the event was handled, FALSE if not
 *
 * Side Effects:    Event-specific actions have been taken.
 *
 * Overview:        This routine is called by the Host layer or client
 *                  driver to notify the application of events that occur.
 *                  If the event is recognized, it is handled and the
 *                  routine returns TRUE.  Otherwise, it is ignored (or
 *                  just "sniffed" and the routine returns FALSE.
 *************************************************************************/

BOOL USB_ApplicationEventHandler ( BYTE address, USB_EVENT event, void *data, DWORD size )
{
    #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
        BYTE i;
    #endif

    // Handle specific events.
    switch ( (INT)event )
    {
        case EVENT_GENERIC_ATTACH:
            if (size == sizeof(GENERIC_DEVICE_ID))
            {
                deviceAddress   = ((GENERIC_DEVICE_ID *)data)->deviceAddress;
                DemoState = BT_STATE_PROCESS; HciState=HCI_CMD_RESET; //YTS !!!!!!!!!!!!!!!!!
                UART2PrintString( "Generic demo device attached - event, deviceAddress=" );
                UART2PutDec( deviceAddress );
                UART2PrintString( "\r\n" );
                #ifdef USB_GENERIC_SUPPORT_SERIAL_NUMBERS
                    for (i=1; i<((GENERIC_DEVICE_ID *)data)->serialNumberLength; i++)
                    {
                        UART2PutChar( ((GENERIC_DEVICE_ID *)data)->serialNumber[i] );
                    }
                #endif
                UART2PrintString( "\r\n" );
                return TRUE;
            }
            break;

        case EVENT_GENERIC_DETACH:
            deviceAddress   = 0;
            DemoState = BT_INITIALIZE;//YTS
            UART2PrintString( "Generic demo device detached - event\r\n" );
            return TRUE;

        case EVENT_GENERIC_TX_DONE:           // The main state machine will poll the driver.
            return TRUE;

        case EVENT_GENERIC_RX1_DONE:

			if(*(int*)data != 0){

				#ifdef DEBUG_MODE
				UART2PrintString( " HCI: " );
				for(data_num=0;data_num<*(int*)data;data_num++)
	      	          {UART2PutHex(buf1[data_num]);UART2PutChar(' ');}
				UART2PrintString( "\r\n" );
				#endif

				if(buf1[0]==0x05) {
					HciState=HCI_CMD_DISCONNECT_REQ;
					hci_acl_init();
				}
				else if(buf1[0]==0x0e || buf1[0]==0x03|| buf1[0]==0x04) hci_acl_init();
				else if(buf1[0]==0x13 && hci_busy_flag==1) {
					hci_busy_flag=0;
					if(acl_busy_flag==0) hid();
				}
			}
            return TRUE;

        case EVENT_GENERIC_RX2_DONE:

			#ifdef DEBUG_MODE
			UART2PrintString( " ACL: " );
			for(data_num=0;data_num<*(DWORD*)data;data_num++)
				{UART2PutHex(buf[data_num]);UART2PutChar(' ');}
			UART2PrintString( "\r\n" );
			#endif

			acl_busy_flag=0;

			if(hci_busy_flag==0) hid();
            return TRUE;

        case EVENT_VBUS_REQUEST_POWER:
            // We'll let anything attach.
            return TRUE;

        case EVENT_VBUS_RELEASE_POWER:
            // We aren't keeping track of power.
            return TRUE;

        case EVENT_HUB_ATTACH:
            UART2PrintString( "\r\n***** USB Error - hubs are not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSUPPORTED_DEVICE:
            UART2PrintString( "\r\n***** USB Error - device is not supported *****\r\n" );
            return TRUE;
            break;

        case EVENT_CANNOT_ENUMERATE:
            UART2PrintString( "\r\n***** USB Error - cannot enumerate device *****\r\n" );
            return TRUE;
            break;

        case EVENT_CLIENT_INIT_ERROR:
            UART2PrintString( "\r\n***** USB Error - client driver initialization error *****\r\n" );
            return TRUE;
            break;

        case EVENT_OUT_OF_MEMORY:
            UART2PrintString( "\r\n***** USB Error - out of heap memory *****\r\n" );
            return TRUE;
            break;

        case EVENT_UNSPECIFIED_ERROR:   // This should never be generated.
            UART2PrintString( "\r\n***** USB Error - unspecified *****\r\n" );
            return TRUE;
            break;

        case EVENT_SUSPEND:
        case EVENT_DETACH:
        case EVENT_RESUME:
        case EVENT_BUS_ERROR:
            return TRUE;
            break;

        default:
            break;
    }

    return FALSE;

} // USB_ApplicationEventHandler


//******************************************************************************
//******************************************************************************
// Main
//******************************************************************************
//******************************************************************************

/*************************************************************************
 * Function:        main
 *
 * Preconditions:   None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Returns:         Never exits
 *
 * Side Effects:    Runs the application
 *
 * Overview:        This is the USB Custom Demo Application's main entry
 *                  point.
 *************************************************************************/

int main ( void )
{
    // Initialize the processor and peripherals.
    if ( InitializeSystem() != TRUE )
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - system.  Halting.\r\n\r\n" );
        while (1);
    }
    if ( USBHostInit(0) == TRUE )
    {
        UART2PrintString( "\r\n\r\n***** USB Custom Demo App Initialized *****\r\n\r\n" );
    }
    else
    {
        UART2PrintString( "\r\n\r\nCould not initialize USB Custom Demo App - USB.  Halting.\r\n\r\n" );
        while (1);
    }

    // Main Processing Loop
    while (1)
    {
        // This demo does not check for overcurrent conditions.  See the
        // USB Host - Data Logger for an example of overcurrent detection
        // with the PIC24F and the USB PICtail Plus.

        // Maintain USB Host State
        USBHostTasks();
        // Maintain Demo Application State
        ManageDemoState();
    }

    return 0;

} // main


/*************************************************************************
 * EOF main.c
 */

